//+------------------------------------------------------------------+
//|                                                   CyberCycle.mq4 |
//|                                                                  |
//| Cyber Cycle                                                      |
//|                                                                  |
//| Algorithm taken from book                                        |
//|     "Cybernetics Analysis for Stock and Futures"                 |
//| by John F. Ehlers                                                |
//|                                                                  |
//|                                              contact@mqlsoft.com |
//|                                          http://www.mqlsoft.com/ |
//+------------------------------------------------------------------+
#property copyright "Coded by Witold Wozniak"
#property link      "www.mqlsoft.com"

#property indicator_separate_window
#property indicator_buffers 2
#property indicator_color1 Red
#property indicator_color2 Blue
#property indicator_level1 0

//
//
//
//
//

extern double Alpha           = 0.07;
extern string note            = "turn on Alert = true; turn off = false";
extern bool   alertsOn        = true;
extern bool   alertsOnCurrent = true;
extern bool   alertsMessage   = true;
extern bool   alertsSound     = true;
extern bool   alertsEmail     = false;
extern string soundfile       = "alert2.wav";

//
//
//
//
//

double Cycle[];
double Trigger[];
double Smooth[];
double P[];
double trend[];

//
//
//
//
//

int init() 
{
    IndicatorBuffers(5);
    SetIndexBuffer(0,Cycle);   SetIndexLabel(0,"Cycle");
    SetIndexBuffer(1,Trigger); SetIndexLabel(1,"Trigger");
    SetIndexBuffer(2,Smooth); 
    SetIndexBuffer(3,P);
    SetIndexBuffer(4,trend);    
    
    IndicatorShortName("Cyber Cycle [" + DoubleToStr(Alpha, 2) + "]");
    return (0);
}

//
//
//
//
//
  
int start() 
{
   int i,limit,counted_bars=IndicatorCounted();

   if(counted_bars<0) return(-1);
   if(counted_bars>0) counted_bars--;
         limit = MathMin(Bars-counted_bars,Bars-1);
   
   //
   //
   //
   //
   //
   
   for(i=limit; i >= 0; i--)
   { 
        P[i]      = (High[i] + Low[i]) / 2.0;   
        Smooth[i] = (P[i] + 2.0 * P[i+1] + 2.0 * P[i+2] + P[i+3]) / 6.0;
        Cycle[i]  = (1.0 - 0.5 * Alpha) * (1.0 - 0.5 * Alpha) * (Smooth[i] - 2.0 * Smooth[i+1] 
                  + Smooth[i+2]) + 2.0 * (1.0 - Alpha) * Cycle[i+1] - (1.0 - Alpha) * (1.0 - Alpha) * Cycle[i+2];
        if (i > Bars - 8) 
        { 
          Cycle[i]   = (P[i] - 2.0 * P[i+1] + P[i+2]) / 4.0; 
        }
          Trigger[i] = Cycle[i+1];
          trend[i] = trend[i+1];
             if (Cycle[i]>Trigger[i]) trend[i] = 1;
             if (Cycle[i]<Trigger[i]) trend[i] =-1;
    }
    
    //
    //
    //
    //
    //
    
    if (alertsOn)
   {
     if (alertsOnCurrent)
          int whichBar = 0;
     else     whichBar = 1;   
     if (trend[whichBar] != trend[whichBar+1])
     if (trend[whichBar] == 1)
           doAlert("buy");
     else  doAlert("sell");       
   }
   return(0);
}

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

void doAlert(string doWhat)
{
   static string   previousAlert="nothing";
   static datetime previousTime;
   string message;
   
      if (previousAlert != doWhat || previousTime != Time[0]) {
          previousAlert  = doWhat;
          previousTime   = Time[0];

          //
          //
          //
          //
          //

          message =  StringConcatenate(Symbol()," at ",TimeToStr(TimeLocal(),TIME_SECONDS)," CyberCycle ",doWhat);
             if (alertsMessage) Alert(message);
             if (alertsEmail)   SendMail(StringConcatenate(Symbol()," CyberCycle "),message);
             if (alertsSound)   PlaySound(soundfile);
      }
}
      
          